#include "config.h"

/*              macro            name          list      */
#define _FONT_ HAVE_FREETYPE { "FreeType", ft_list_fonts },

#ifdef HAVE_FREETYPE
  
#ifdef HAVE_STDLIB_H
# include <stdlib.h>
#endif

#ifdef HAVE_UNISTD_H
# include <unistd.h>
#endif

#ifdef HAVE_SYS_TYPES_H
# include <sys/types.h>
#endif

#include <dirent.h>

#include <stdio.h>

#include <freetype/freetype.h>

#include "libsub_private.h"

#include "math.h"
  
static FT_Library  library;

struct tt_face
{
  struct tt_font *font;
  int size;
  int flags;

  struct image *(*draw)( struct face *,unsigned short *,int,int,int,int);
  FT_Face        face;
};

struct tt_private
{
  struct tt_face fp;
  char          *fname;
};

struct tt_font
{
  char *name;
  int flags;
  struct face *(*open)(struct font *, int, int );

  struct tt_private *private;
};

static void blit_character( struct image *i, FT_GlyphSlot slot,
			    struct argb color,
			    int xp, int yp, int shf,
			    int fl )
{
  struct argb   *ii = i->data  + xp + yp * i->w;
  unsigned char *sa;
  int x=0, y=0;

  if( !slot->bitmap.rows || !slot->bitmap.width )
    return;
  
  if( slot->bitmap.pitch < 0 )
    sa = slot->bitmap.buffer - slot->bitmap.pitch*y;
  else
    sa = slot->bitmap.buffer;

  for( y = 0; y < slot->bitmap.rows; y++ )
  {
    if( yp + y >= i->h ) return;
    if( yp + y < 0 ) continue;

    for( x = 0; x<slot->bitmap.width; x++ )
    {
      if( xp + x >= i->w ) continue;
      if( xp + x < 0 )     continue;
      if( (color.a = (sa[x]>>shf)) ) // source has value.
	if( ii[x].a ) // destination also has value
	{
	  ii[x].a = MINIMUM((color.a+ii[x].a),255);
	  if( !fl ) // Otherwise the color is already correct.
	  {
	    float a  =  color.a * 1/255.0;
	    float a1 = 1.0-a;
	    ii[x].r=(unsigned char) ((int)(a1 * ii[x].r) + (int)(a * color.r));
	    ii[x].g=(unsigned char) ((int)(a1 * ii[x].g) + (int)(a * color.g));
	    ii[x].b=(unsigned char) ((int)(a1 * ii[x].b) + (int)(a * color.b));
	  }
	}
	else
	  ii[x] = color;
    }
    ii += i->w;
    sa += slot->bitmap.pitch;
  }
}

static struct image *_ft_draw( struct face *fa,
			       unsigned short *text,
			       int color, int ocolor, int scolor )
{
  FT_Face f = ((struct tt_face *)fa)->face;
  int baseline, w=0, h, x0=0, x00, i, pass;
  int od;
  struct image *img;
  struct argb  filled;
  FT_GlyphSlot  slot = f->glyph; // minor optimization.

  FT_Set_Pixel_Sizes( f, 0, fa->size );

  baseline = (f->size->metrics.height+f->size->metrics.descender)>>6;
  h=f->size->metrics.height>>6;
  
  for( i=0; text[i]; i++ )
  {
    if( text[i] == ' ' )
      w += MAXIMUM( fa->size/4, 2 );
    else if(!FT_Load_Glyph( f, FT_Get_Char_Index( f, text[i] ), FT_LOAD_RENDER ))
      w += (slot->advance.x>>6);
  }
/*    w += MAXIMUM( 0, slot->bitmap_left+slot->bitmap.width ); */
  
  od = MAXIMUM( 1, h/12 );
  
  if( fa->flags & FONT_SHADOW )
  {
    h+=MAXIMUM(od*2,3);
    w+=MAXIMUM(od*2,3);
  }
  if( fa->flags & FONT_OUTLINE )
  {
    h+=od*2;
    w+=od*2;
    x0+=od;
    baseline+=od;
  }

  x00 = x0;
  img = allocate_image( w, h );
  
# define COLOR(X,Y) X.r=((Y)>>16)&255;X.g=((Y)>>8)&255;X.b=(Y)&255;X.a=0;


#define RENDER(X0,Y0,SHF) blit_character(img,slot,filled,X0+x0+MAXIMUM(slot->bitmap_left,0),Y0+baseline-slot->bitmap_top,SHF,0);
#define RENDERN(X0,Y0,SHF) blit_character(img,slot,filled,X0+x0+MAXIMUM(slot->bitmap_left,0),Y0+baseline-slot->bitmap_top,SHF,1);


  for( pass = ((fa->flags&FONT_SHADOW)?0:(fa->flags&FONT_OUTLINE)?1:2);
       pass<3; pass+=(fa->flags&FONT_OUTLINE)?1:2 )
    for( x0=x00,i=0; text[i]; i++ )
    {
      if( text[i] == ' ' )
	x0 += MAXIMUM( fa->size/4, 2 );
      else
      {
	if(!FT_Load_Glyph( f, FT_Get_Char_Index(f,text[i] ), FT_LOAD_RENDER))
	{
	  if( (pass == 0) && (fa->flags & FONT_SHADOW) )
	  {
	    int d = MAXIMUM(od/2,1);
	    int z = MAXIMUM(od*2,3);
	    COLOR(filled,scolor);
	    /* soft shadow. */
	    RENDERN( z,z-d,3 );
	    RENDERN( z-d,z,3 );    RENDERN( z+d,z,3 );
	    RENDERN( z,z+d,3 );
	    /* main shadow */
	    RENDERN( z,z,2 );
	  }
	  else if( (pass == 1 ) && (fa->flags & FONT_OUTLINE) )
	  {
	    float i;
	    COLOR(filled,ocolor);
	    /* The bigger the outline, the better it has to be. */
	    if( od > 1 )
	    {
	      int ox=0,oy=0;
	      for( i = 0.0; i<3.1415*2; i+=0.3/od )
	      {
		int x = rint(od*sin( i ));
		int y = rint(od*cos( i ));
		if( x && y && (x!=ox || y!=oy) )
		{
		  ox = x;
		  oy = y;
		  RENDERN(rint(x),rint(y),2);
		}
	      }
	    }
	    else
	    {
	      RENDERN(0,-1,0);
	      RENDERN(-1,0,0); 	      RENDERN(1,0,0);
	      RENDERN(0,1,0);

	    }
	  }
	  else if( (pass == 2 ) )
	  {
	    COLOR(filled,color);
	    RENDER(0,0,0);
	  }
	}
	x0 += slot->advance.x>>6;
      }
    }
  return img;
}
			       


static struct face *_ft_open( struct font *f, int size, int flags )
{
  struct tt_private *p = ((struct tt_font *)f)->private;
  struct tt_face *face = &p->fp;
  
  face->font  = (void *)f;
  face->flags = flags;
  face->draw  = (void *)_ft_draw;

  if( face->size )
  {
    face->size  = size;
    return (struct face *)face;
  }

  face->size  = size;
  FT_New_Face( library, p->fname, 0, &face->face );
  return (struct face *)face;
}

static struct font *_ft_font( char *path )
{
  struct tt_font *f = (struct tt_font *)allocate_font();
  FT_Face face;
  f->private = xalloc( sizeof( struct tt_private ) );

  f->private->fname = (char *)xstrdup( path );
  f->private->fp.size = 0;
  
  if( FT_New_Face( library, path, 0, &face ) )
  {
    xfree( f );
    return 0;
  }
  f->name = (char *)xstrdup( face->family_name );

  if( face->style_flags & FT_STYLE_FLAG_BOLD )
    f->flags |= FONT_BOLD;

  if( face->style_flags & FT_STYLE_FLAG_ITALIC )
    f->flags |= FONT_ITALIC;

  f->open = _ft_open;

  FT_Done_Face( face );
  
  return (struct font *)f;
}



struct font **ft_list_fonts( char **directories )
{
  static int fs_size = 64, nfonts = 0;
  static struct font **font_list;
  int i;

  if( !font_list )
    font_list = xalloc( sizeof( struct font *) * fs_size );
  else
    return font_list;
  
  FT_Init_FreeType( &library );

  font_list[0] = 0;

  for( i=0; directories[i]; i++ )
  {
    struct dirent *e;
    DIR *d;
    char fn[1024];
    char *p;
    if( strlen( directories[i] ) > 500 )
      continue;
    strcpy( fn, directories[i] );
    strcat( fn, "/" );
    p = fn + strlen( directories[i] ) + 1;
    d = opendir( directories[i] );
    if( !d )
      continue;
    
    while( (e = readdir(d)) )
    {
      struct font *f;
      strcpy( p, e->d_name );
      if( (f = _ft_font( fn )) )
      {
	font_list[ nfonts++ ] = f;
	if( nfonts == fs_size )
	{
	  font_list = realloc( font_list, sizeof( struct font *)*(fs_size+32));
	  fs_size+=32;
	}
      }
    }
    closedir( d );
  }
  font_list[nfonts]=0;
  return font_list;
}
#endif
