#include "config.h"
#ifdef HAVE_STDLIB_H
# include <stdlib.h>
#endif

#ifdef HAVE_UNISTD_H
# include <unistd.h>
#endif

#ifdef HAVE_SYS_TYPES_H
# include <sys/types.h>
#endif

#include <stdio.h>
#include <dirent.h>
#include <string.h>
#include <libgen.h>

#include "libsub_private.h"

static struct sub_handler 
{
  char *name;
  int (*identify)();
  struct subtitle **(*read)();
  char *exts[4];           /* case insensitive */
}
handlers[] =
{
#include "formats.h"
};

struct subtitle **lsub_read_subtitles( FILE *f )
{
  int i;
  libsubFILE *lsf = lsub_freopen( f );
  struct subtitle **res = NULL;
  for( i = 0; i<(int)NELEM( handlers ); i++ )
    if( handlers[i].identify( lsf ) )
    {
      fprintf( stderr, "Reading %s subtitles.\n", handlers[i].name );
      res = handlers[i].read( lsf );
      break;
    }
  lsf->fclose( lsf );
  return res;
}

struct subtitle **lsub_read_subtitles_memory( void *d, int l )
{
  int i;
  libsubFILE *lsf = lsub_memopen( d,l );
  struct subtitle **res = NULL;
  for( i = 0; i<(int)NELEM( handlers ); i++ )
    if( handlers[i].identify( lsf ) )
    {
      fprintf( stderr, "Reading %s subtitles.\n", handlers[i].name );
      res = handlers[i].read( lsf );
      break;
    }
  lsf->fclose( lsf );
  return res;
}

struct subtitle **lsub_use_subtitlefile( char *subfile )
{
  FILE *f = fopen( subfile, "r" );
  if( !f )
  {
    fprintf( stderr, "Found %s, but it cannot be opened\n", subfile );
    return 0;
  }
  return lsub_read_subtitles( f );
}

struct subtitle **lsub_find_subtitles_for( char *filename )
{
  DIR *d;
  char *p;
  struct subtitle **res;
#ifdef PATH_MAX
  char fn[PATH_MAX], tmp[PATH_MAX];
#else
  char fn[MAXIMUM(pathconf(filename,_PC_PATH_MAX),4096)],
      tmp[MAXIMUM(pathconf(filename,_PC_PATH_MAX),4096)]; // Note: gcc only...
#endif
  int i;
  
  strcpy( tmp, filename );
  strcat( tmp, "/../" );

  if( !realpath( tmp, fn ) )
  {
    perror("Cannot resolve path");
    return 0;
  }



  strcat( tmp, filename );
  p = xstrdup( basename( tmp ) );

  for( i=strlen(p)-1; i>0; i-- )
    if( p[i] == '.' )
    {
      p[i]=0;
      break;
    }
  
  for( i = 0; i<(int)NELEM( handlers ); i++ )
  {
    int j;
    struct dirent *e;
    d = opendir( fn );
    if( !d )
    {
      perror("Cannot open directory");
      xfree( p );
      return 0;
    }
    
    while( (e = readdir(d)) )
    {
      for( j = 0; j<4; j++ )
	if( handlers[i].exts[j] )
	{
	  strcpy( tmp, p );
	  strcat( tmp, "." );
	  strcat( tmp, handlers[i].exts[j] );
	  if( !strcasecmp( e->d_name, tmp ) )
	  {
	    // We have a winner!
	    strcpy( tmp, fn );
	    strcat( tmp, "/" );
	    strcat( tmp, e->d_name );
	    if( (res = lsub_use_subtitlefile( tmp )) )
	    {
	      closedir( d );
	      xfree( p );
	      return res;
	    }
	  }
	}
    }
    closedir( d );
  }
  xfree( p );
  return 0;
}
