/* srt support */

#include "config.h"

#ifdef HAVE_STDLIB_H
# include <stdlib.h>
#endif

#ifdef HAVE_UNISTD_H
# include <unistd.h>
#endif

#ifdef HAVE_SYS_TYPES_H
# include <sys/types.h>
#endif

#include <stdio.h>
#include <string.h>

#include "libsub_private.h"


/*                name  identify     parse      valid exts. */
#define _FORMAT_ {"srt",srt_identify,srt_parse,{"srt",0,0,0}},

/* .srt file format example:
6
00:00:19,921 --> 00:00:22,822
Assuming the enemy
is an outsider,
we can't know for sure...

7
... */

static int srt_read_line( libsubFILE *fd, struct subtitle *current )
{
  char line[1001];
  int allocated_size = 0;
  int sh, sm, ss, sms;
  int eh, em, es, ems;

  if( !fd->fgets( line, 1000, fd ) ) return 0; /* uninteresting line number */
  if( !fd->fgets( line, 1000, fd ) ) return 0; /* start / end timing */
  if( sscanf(line, "%d:%d:%d%*[,.:]%d --> %d:%d:%d%*[,.:]%d",
	     &sh, &sm, &ss, &sms,
	     &eh, &em, &es, &ems) < 8 )
    return 0;
  current->start = sh*360000 + sm*6000 + ss*100 + sms/10;
  current->end   = eh*360000 + em*6000 + es*100 + ems/10;

  while( fd->fgets( line, 1000, fd ) )
  {
    if( !line[0] || line[0] == '\n' || line[0] == '\r' )
      return 1;
    if( current->ntext >= allocated_size )
    {
      allocated_size += 10;
      current->text = realloc( current->text,
			       allocated_size*
			       sizeof(current->text[0]) );
    }
    memset( &current->text[current->ntext],0,
	    sizeof(current->text[current->ntext]) );
    int l = strlen( line );
    if( line[l-1] == '\n' ) l--;
    if( line[l-1] == '\r' ) l--;
    line[l] = 0;
    current->text[ current->ntext].nl = 1;
    current->text[ current->ntext++ ].str = sub_iso8859_to_unicode( line );
  }
  return 1;
}


int srt_identify( libsubFILE *fd )
{
  char line[1000];
  int j, i;
  fd->rewind( fd );

  for( j = 0; j<3; j++ )
  {
    if( !fd->fgets( line, 1000, fd ) )
      return 0;
    if( sscanf(line, "%d:%d:%d%*[,.:]%d --> %d:%d:%d%*[,.:]%d",
	       &i, &i, &i, &i, &i, &i, &i, &i) >= 8 )
    {
      fd->rewind( fd );
      return 1;
    }
  }
  return 0;
}

#define PFUN      srt_parse
#define LFUN      srt_read_line
#define CLEANUP()
#define SORTED
#include "sub_parse_function.h"
