#ifndef HAVE_LIBSUB_H
#define HAVE_LIBSUB_H
struct subtitle;

struct argb {  unsigned char r, g, b, a; };

struct image 
{
  int w, h;
  struct argb *data;
};


/* Allocate an image to render subtitles into. The format is always
 * 'argb' right now. Conversion functions will be added.
 */
struct image *lsub_allocate_image( int w, int h );

/* Free an image previously allocated by allocate_image */
void lsub_free_image( struct image *i );

/* Set the font directory list. This should be specified in some sort
 * of configuration file or taken from X; I guess.
 */
void lsub_set_font_directories( char ** directories );

/* Try to find subtitles for the specified file. The search is done
 * case-insensitive with all exts. specified by the format modules.
 *
 * As an example, works with the given filename
 * card_captor_sakura_movie_2.avi and the subtitlefile
 * Card_Captor_Sakura.SSA, '_'/' ' and '-' are not equivalent at this
 * time, though.
 */
struct subtitle **lsub_find_subtitles_for( char *filename );

/* Read subtitles from the specified file. If no parser can be found
 * that can parse the contents of the time, return 0.
 */
struct subtitle **lsub_use_subtitlefile( char *subfile );

/* The lowest level subtitle reader: Use the specified FILE *
 */
struct subtitle **lsub_read_subtitles( FILE *f );
struct subtitle **lsub_read_subtitles_memory( void *d, int l );

/* From the array of subtitles, find the ones that should be shown at
 * current_time, and if they differ from the last rendered frame,
 * render them into screen and return 1. Otherwise, return 0.
 *
 * If 0 is returned, the contents of 'screen' will be left untouched.
 */
int lsub_render_subtitles( struct subtitle **subs, int current_time,
			   struct image *screen );


void lsub_free_subtitles( struct subtitle **subs );
#endif
