#include "config.h"

#ifdef HAVE_STDLIB_H
# include <stdlib.h>
#endif

#ifdef HAVE_UNISTD_H
# include <unistd.h>
#endif

#ifdef HAVE_SYS_TYPES_H
# include <sys/types.h>
#endif

#include <stdio.h>
#include <string.h>

#include "libsub_private.h"


/* File handling abstraction layer */

/* FILE * wrappers */
static char *_f_fgets( char *p, int n, libsubFILE *fd )
{
  return fgets( p, n, (FILE *)fd->p );
}

static void _f_frewind( libsubFILE *fd )
{
  return rewind( (FILE *)fd->p );
}

static void _f_fclose( libsubFILE *fd )
{
  fclose( (FILE *)fd->p );
  free( fd );
}

libsubFILE *lsub_freopen( FILE *f )
{
  libsubFILE *fd = malloc( sizeof( libsubFILE ) );
  fd->p = f;
  fd->fgets = _f_fgets;
  fd->rewind = _f_frewind;
  fd->fclose = _f_fclose;

  return fd;
}

/* Memory file */
struct memfile
{
  char *data;
  unsigned int len;
  unsigned int pos;
};


static char *_m_fgets( char *res, int n, libsubFILE *fd )
{
  struct memfile *m = (struct memfile*)fd->p;
  int p = 0;
  if( m->pos >= m->len )
    return 0;
  while( (p<(n-1)) && (m->pos < m->len) &&
	 (m->data[ m->pos ] != '\n' &&
	  m->data[ m->pos ] != '\r' ) )
    res[p++] = m->data[m->pos++];
  if( (m->pos < m->len) && ((m->data[m->pos] == '\r') )) {
    res[p++]='\r';
    m->pos++;
  }
  if( (m->pos < m->len) && ((m->data[m->pos] == '\n') )) {
    res[p++]='\n';
    m->pos++;
  }
  res[p] = 0;
  return res;
}

static void _m_frewind( libsubFILE *fd )
{
  ((struct memfile*)fd->p)->pos = 0;
}

static void _m_fclose( libsubFILE *fd )
{
  free( fd );
}

libsubFILE *lsub_memopen( char *d, int l )
{
  libsubFILE *fd = malloc( sizeof( libsubFILE ) + sizeof( struct memfile ));
  struct memfile *m = (struct memfile *)(((void *)fd)+sizeof( libsubFILE ) );
  fd->p = m;
  fd->fgets = _m_fgets;
  fd->rewind = _m_frewind;
  fd->fclose = _m_fclose;
  m->data = d;
  m->len = l;
  m->pos = 0;
  return fd;
}
