#include "libsub.h"

#define FONT_SHADOW  1
#define FONT_OUTLINE 2
#define FONT_ITALIC  4
#define FONT_BOLD    8

/* Color changed in part. */
#define FONT_COLOR  256

#define ALIGN_LEFT   1
#define ALIGN_RIGHT  2
#define ALIGN_TOP    4
#define ALIGN_CENTER 8

/* #define DEBUG_MALLOC */


#ifndef DEBUG_MALLOC
# define xalloc(X) malloc(X)
# define xfree(X)  free(X)
# define xstrdup(X)  strdup(X)
#else
void *xalloc( size_t s );
char *xstrdup( char *x );
void xfree( void *d );
#endif

#ifndef HAVE_STRSEP
char *strsep( char **stringp, const char *delim );
#endif

struct font
{
  char *name;
  int flags;
  struct face *(*open)(struct font *, int, int );
  void *private;
};

struct face
{
  struct font *font;
  int size;
  int flags;
  struct image *(*draw)(struct face *,unsigned short *, int, int, int );
  void *private;
};

struct subtitle
{
  unsigned char at;
  unsigned char flags;                 // bold etc.
  unsigned short ntext;                // number of segments.

  int start, end;

  float hm;                  // left/right margin
  float vm;                  // top/bottom margin

  float size;                // 1.0 == screen height.
  int color,scolor,ocolor;   // main, shadow, outline

  struct part
  {
    unsigned short    nl;
    unsigned short    delay; // in 1/100 seconds
    unsigned short    flags, nflags;

    int    color; 
    struct font *font;
                       
    short *str; 
  } *text;
};

typedef struct _lsFILE
{
  void *p;
  char *(*fgets)( char *, int , struct _lsFILE * );
  void (*rewind)( struct _lsFILE * );
  void (*fclose)( struct _lsFILE * );
} libsubFILE;

libsubFILE *lsub_freopen( FILE *f );
libsubFILE *lsub_memopen( char *d, int l );

struct font *allocate_font( );
struct face *allocate_face( );

struct image *allocate_image( int w, int h );
void free_image( struct image *i );

struct font **ft_list_fonts( char **directories );

struct font *lsub_find_font( char *name, int flags );

int ssa_identify( libsubFILE *f );
struct subtitle **ssa_parse( libsubFILE *f );

int srt_identify( libsubFILE *f );
struct subtitle **srt_parse( libsubFILE *f );

int udvd_identify( libsubFILE *f );
struct subtitle **udvd_parse( libsubFILE *f );

unsigned short *sub_iso8859_to_unicode( unsigned char *c );

