#include "config.h"

#ifdef HAVE_STDLIB_H
# include <stdlib.h>
#endif

#ifdef HAVE_UNISTD_H
# include <unistd.h>
#endif

#ifdef HAVE_SYS_TYPES_H
# include <sys/types.h>
#endif

#include <stdio.h>
#include "libsub_private.h"

#ifdef DEBUG_MALLOC

void *xalloc( size_t size )
{
  unsigned char *real = malloc( size+4096 );
  memset( real, 0xfa, size+4096 );
  ((unsigned int *)real)[0] = size + 4096;
  ((unsigned int *)real)[1] = (unsigned int)real;
  ((unsigned int *)real)[2] = (unsigned int)(real+2048);
  ((unsigned int *)real)[3] = (unsigned int)(real+2048+size);
  fprintf( stderr, "malloc %d -> %p..%p..%p\n", size,
	   real, real+2048, real+4096+size );
  return real + 2048;
}

char *xstrdup( char *x )
{
  int   n = strlen( x );
  char *t = xalloc( n+1 );
  strcpy( t, x );
  return t;
}

void xfree( void *x )
{
  unsigned char *real = x-2048;
  int size, i;

  if( (int)x < 4096 )
  {
    fprintf( stderr, "Calling free( %p )\n", x );
    abort();
  }

  if( ((unsigned int *)real)[1] != (unsigned int)real )
  {
    fprintf( stderr,
	     "Illegal pointer in memblock, expected %p; was %p\n",
	     (void *)((unsigned int)real),
	     (void *)(((unsigned int *)real)[1]) );
    abort();
  }

  if( (size = ((unsigned int *)real)[0]) < 4096 )
  {
    fprintf( stderr,
	     "Illegal size in memblock, was %d; expected at least 4096\n",
	     size );
    abort();
  }

  fprintf( stderr, "free %d -> %p..%p..%p\n", size-4096, real,real+2048,real+size );

  for( i = 16; i<2048; i++ )
    if( real[i] != 0xfa )
    {
      fprintf( stderr, "Buffer underrun in %p at offset -%d\n",
	       real, 2048-i );
      abort();
    }
  for( i = size-2048; i<size; i++ )
    if( real[ i ] != 0xfa )
    {
      fprintf( stderr,
	       "Buffer overrun in %p at offset %d from end\n",
	       real, i-(size-2048) );
      abort();
    }
  free( real );
}
#endif
