#include "config.h"

#ifdef HAVE_STDLIB_H
# include <stdlib.h>
#endif

#ifdef HAVE_UNISTD_H
# include <unistd.h>
#endif

#ifdef HAVE_SYS_TYPES_H
# include <sys/types.h>
#endif

#include <stdio.h>

#include <string.h>

#include "libsub_private.h"

static inline void blit( struct image *screen,
			 int x, int y,
			 struct image *img )
{
  int i, j;
  int ww = MINIMUM( img->w, screen->w-x );
  int hh = MINIMUM( img->h, screen->h-y );
  struct argb *sp = screen->data + x+y*screen->w;
  struct argb *ip = img->data;

  if( x < 0 )
  {
    sp += -x;
    ip += -x;
    ww += x;
  }

  if( y < 0 )
  {
    sp += -y * screen->w;
    ip += -y * img->w;
    hh += y;
  }

  for( i = 0; i<hh; i++ )
  {
    for( j = 0; j<ww; j++ )
    {
      if( ip[ j ].a  )
	sp[ j ] = ip[ j ];
    }
    sp += screen->w;
    ip += img->w;
  }
}

static int draw_one_sub( struct subtitle *s, struct image *screen,
			 int last_y_bot, int into )
{
  int sz = MAXIMUM( (int)(s->size * screen->h), 12 );
  struct face *f = 0;
  struct font *of = 0;

  /* FIXME: this should _really_ be allocated on demand!
   * The stackspace wastage here is quite terrible.
   *
   * Also, this enforces arbitrary limits on things.
   */
#define SUB_MAX_ROWS 20
#define SUB_MAX_WORDS 60
  short nl[ SUB_MAX_ROWS ][ SUB_MAX_WORDS+2 ];
  struct image *lines[ SUB_MAX_ROWS*SUB_MAX_WORDS ];

  int i, j, l=0, p=1;
  int h = screen->h;
  int xx, yy, th;

  memset( nl, 0, sizeof(nl) );

  for( i = 0; i<s->ntext; i++ )
  {
    int c = s->color;

    if( !s->text[i].str ) /* null event */
      continue;

    if( s->text[i].font && s->text[i].font != of )
    {
      of = s->text[ i ].font;
      f = of->open( of, sz,
		    ((s->flags|s->text[i].flags)&~s->text[i].nflags) );
    }
    if( s->text[i].flags & FONT_COLOR )
      if( into > s->text[i].delay )
	c = s->text[i].color;
    lines[i] = f->draw(f, s->text[i].str, c, s->ocolor, s->scolor );

    if( nl[l][0] + lines[i]->w > screen->w )
    {
      l++;
      p = 1;
    }
    nl[l][p] =  i;
    nl[l][++p]=-1;
    nl[l][0] += lines[i]->w;
    
    if( s->text[i].nl )
    {
      l++;
      p = 1;
    }
  }
  th = sz * l;
  lines[i] = 0;

  
  if( s->at & 4 ) // top
    yy = (s->vm*h);
  else if( s->at & 8 ) // center
    yy = (h-th)>>1;
  else
  {
    h = last_y_bot;
    last_y_bot = h-th;
    yy = (h-s->vm*h)-th;
  }
  
  for( j = 0; nl[j][0]; j++ )
  {
    int p;
    switch( s->at & 3 )
    {
      case 0: // center
	xx = (screen->w-nl[j][0])>>1;
	break;

      case 1: // left
	xx = s->hm*screen->w;
	break;
	
      default:
	xx = (screen->w- s->hm*screen->w) - nl[j][0];
	break;
    }

    for(  p = 1; (l=nl[j][p]) != -1; p++ )
    {
      blit( screen, xx, yy-lines[ l ]->h, lines[ l ] );
      xx += lines[ l ]->w;
      free_image( lines[l] );
    }
    yy += sz;
  }
  return last_y_bot;
}

int lsub_render_subtitles( struct subtitle **subs, int current_time,
			   struct image *screen )
{
  static struct subtitle **old_subs;
  static int current_sub;
  static int onscreen[ SUB_MAX_ROWS ];
  static int old_nsubs;

  int i, j=0, last_y_bot = screen->h;

  if( subs != old_subs )
  {
    current_sub = 0;
    memset(onscreen, 0, sizeof( onscreen ) );
    old_nsubs = 0;
    old_subs = subs;
  }

  /* Mostly we only need to step at most one step forwards or
     backwards, thus no binary search, even if it would have been
     rather easy to add.

     When I tried one, it was (slightly) slower
  */

  while( current_sub>0 && subs[current_sub] &&
	 (subs[current_sub]->start > current_time) )
    current_sub--;

  while( subs[current_sub] && (subs[current_sub]->end < current_time) )
    current_sub++;

  if( !subs[current_sub] )
  {
    if( current_sub )
      current_sub--;
    if( old_nsubs )
    {
      memset( screen->data, 0, screen->w*screen->h*4 );
      old_nsubs = 0;
      return 1;
    }
    return 0;
  }

  // Pass 1: Check if the screen should be changed.
  i = current_sub;j=0;
  while( subs[i] && subs[i]->start < current_time )
  {
    if( subs[i]->end > current_time )
    {
      if( j == SUB_MAX_ROWS )
	return 0;
      if( onscreen[j++] != i )
	goto gogogo;
    }
    i++;
  }

  if( !subs[i] || j == old_nsubs )
    return 0;
gogogo:
  memset( screen->data, 0, screen->w*screen->h*4 );

  /* FIXME: Order subs by priority */
  i = current_sub; j=0;
  while( subs[i] && subs[i]->start < current_time )
  {
    if( subs[i]->end > current_time )
    {
      if( j == SUB_MAX_ROWS )
	return 1;
      last_y_bot = draw_one_sub( subs[i], screen, last_y_bot, current_time-subs[i]->start );
      onscreen[j++] = i;
    }
    onscreen[ j ] = -1;
    i++;
  }
  old_nsubs = j;
  return 1;
}
